/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.resources;

import ca.uhn.fhir.model.api.annotation.Child;
import ca.uhn.fhir.model.api.annotation.ResourceDef;
import org.hl7.fhir.exceptions.FHIRException;
import org.hl7.fhir.r4.model.Base;
import org.hl7.fhir.r4.model.DomainResource;
import org.hl7.fhir.r4.model.Reference;
import org.hl7.fhir.r4.model.ResourceType;
import org.hl7.fhir.r4.model.StringType;

import static com.example.resources.Meal.EMPTY_BASE_ARRAY;

@ResourceDef(name = "MealIngredient")
public class MealIngredient extends DomainResource {
	@Child(name = "name")
	private StringType myName;

	@Child(
			name = "meal",
			type = {Meal.class})
	private Reference myMeal;

	public String getName() {
		return myName != null ? myName.getValue() : null;
	}

	public void setName(String theName) {
		myName = theName != null ? new StringType(theName) : null;
	}

	public Reference getMeal() {
		return myMeal;
	}

	public void setMeal(Reference theMeal) {
		myMeal = theMeal;
	}

	/**
	 * All resource classes must implement the {@literal copy()} method.
	 */
	@Override
	public DomainResource copy() {
		MealIngredient retVal = new MealIngredient();
		super.copyValues(retVal);
		retVal.myName = myName != null ? myName.copy() : null;
		retVal.myMeal = myMeal != null ? myMeal.copy() : null;
		return retVal;
	}

	/**
	 * This method is called by the FHIRPath evaluator when evaluating
	 * SearchParameters.
	 */
	@Override
	public Base[] getProperty(int theHash, String theName, boolean theCheckValid) throws FHIRException {
		switch (theName) {
			case "meal":
				return myMeal != null ? new Base[] {myMeal} : EMPTY_BASE_ARRAY;
			case "name":
				return myName != null ? new Base[] {myName} : EMPTY_BASE_ARRAY;
			default:
				return super.getProperty(theHash, theName, theCheckValid);
		}
	}

	@Override
	public String fhirType() {
		return "MealIngredient";
	}

	@Override
	public ResourceType getResourceType() {
		return null;
	}
}
